﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace SignData
{
    public class DataValidation
    {
        public static string SignData(string privateKey, string data)
        {
            try
            {
                var bData = Encoding.UTF8.GetBytes(data);
                var rsa = GetRSACryptoServiceProviderByPrivateKey(privateKey);
                if (rsa == null)
                {
                    throw new Exception($"Không khởi tạo được RSACryptoServiceProvider");
                }
                SHA1Managed hash = new SHA1Managed();
                byte[] hashedData = hash.ComputeHash(bData);
                return Convert.ToBase64String(rsa.SignHash(hashedData, CryptoConfig.MapNameToOID("SHA1")));
            }
            catch
            {
                throw;
            }
        }

        public static bool VerifySignData(string publicKey, string data, string checkSum)
        {
            try
            {
                byte[] bSign = Convert.FromBase64String(checkSum);
                byte[] bSource = Encoding.UTF8.GetBytes(data);

                var rsa = GetRSACryptoServiceProviderByPublicKey(publicKey);
                if (rsa == null)
                {
                    throw new Exception($"VerifySignData - Xác thực không hợp lệ! - rsa null");
                }

                return rsa.VerifyData(bSource, CryptoConfig.MapNameToOID("SHA1"), bSign);
            }
            catch
            {
                throw;
            }
        }

        public static RSACryptoServiceProvider GetRSACryptoServiceProviderByPrivateKey(string privateKey)
        {
            try
            {
                RSACryptoServiceProvider rsaCSP = new RSACryptoServiceProvider();
                if (privateKey.StartsWith("-----BEGIN RSA PRIVATE KEY-----") && privateKey.EndsWith("-----END RSA PRIVATE KEY-----"))
                {
                    rsaCSP.ImportRSAPrivateKey(Convert.FromBase64String(privateKey.Replace("-----BEGIN RSA PRIVATE KEY-----", "").Replace("-----END RSA PRIVATE KEY-----", "")), out _);
                }
                else
                {
                    throw new Exception($"Private key không hợp lệ");
                }
                return rsaCSP;
            }
            catch
            {
                throw;
            }
        }

        public static RSACryptoServiceProvider GetRSACryptoServiceProviderByPublicKey(string publicKey)
        {
            try
            {
                RSACryptoServiceProvider rsaCSP = new RSACryptoServiceProvider();
                if (publicKey.StartsWith("-----BEGIN RSA PUBLIC KEY-----") && publicKey.EndsWith("-----END RSA PUBLIC KEY-----"))
                {
                    rsaCSP.ImportRSAPublicKey(Convert.FromBase64String(publicKey.Replace("-----BEGIN RSA PUBLIC KEY-----", "").Replace("-----END RSA PUBLIC KEY-----", "")), out _);
                }
                else if (publicKey.StartsWith("-----BEGIN PUBLIC KEY-----") && publicKey.EndsWith("-----END PUBLIC KEY-----"))
                {
                    rsaCSP.ImportSubjectPublicKeyInfo(Convert.FromBase64String(publicKey.Replace("-----BEGIN PUBLIC KEY-----", "").Replace("-----END PUBLIC KEY-----", "")), out _);
                }
                else
                {
                    throw new Exception("PublicKey không hợp lệ");
                }
                return rsaCSP;
            }
            catch
            {
                throw;
            }
        }
    }
}
